package de.dhbwloe.campusapp.fragments;


import android.os.Bundle;
import android.support.v4.app.Fragment;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.ListView;
import android.widget.ProgressBar;
import android.widget.TextView;

import java.util.ArrayList;
import java.util.Date;

import de.dhbwloe.campusapp.CampusAppContext;
import de.dhbwloe.campusapp.CampusAppFragment;
import de.dhbwloe.campusapp.R;
import de.dhbwloe.campusapp.search.SearchIndices;
import de.dhbwloe.campusapp.search.SearchTarget;

/**
 * A simple {@link Fragment} subclass.
 */
public class AppSearchInternal extends CampusAppFragment implements AppSearchProvider {
    private String sSearchQuery;
    private boolean bQueryExecuted = false;
    private AppSearchListAdapter appSearchAdapter;
    private ArrayList<AppSearchListItem> searchResultItems = new ArrayList<AppSearchListItem>();

    @Override
    public void onCreate(Bundle savedInstanceState) {
        Log.i("AppSearchInternal", "Event: onCreate");
        super.onCreate(savedInstanceState);
        Bundle savedState;
        if((savedState = getArguments()) != null)
            sSearchQuery = savedState.getString("query");
        else if(savedInstanceState != null && (savedState = savedInstanceState.getBundle("SearchInternal")) != null) {
            sSearchQuery = savedState.getString("query");
            if(savedState.getBoolean("executed")) {
                executeSearch(false);
            }
        }
    }

    @Override
    public void onSaveInstanceState(Bundle savedInstanceState) {
        Log.i("AppSearchInternal", "Event: onSaveInstanceState");
        Bundle bundle = new Bundle();
        bundle.putString("query", sSearchQuery);
        bundle.putBoolean("executed", bQueryExecuted);
        savedInstanceState.putBundle("SearchInternal", bundle);
        super.onSaveInstanceState(savedInstanceState);
    }

    private void parseIstanceState(Bundle bundle) {
        if(sSearchQuery != null)
            return;
        Bundle args = getArguments();
        if(args != null) {
            String queryString = args.getString("query");
            if(queryString != null)
                sSearchQuery = queryString;
        }
        if(sSearchQuery == null && bundle != null) {
            String queryString = bundle.getString("query");
            if(queryString != null)
                sSearchQuery = queryString;
        }
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        Log.i("AppSearchInternal", "Event: onCreateView");
        if(AppContext == null)
            AppContext = CampusAppContext.getInstance();
        oFragmentView = inflater.inflate(R.layout.fragment_appsearch_list, container, false);

        parseIstanceState(savedInstanceState);

        ListView searchResultList = (ListView) oFragmentView.findViewById(R.id.searchResultItems);
        appSearchAdapter = new AppSearchListAdapter(oFragmentView.getContext(), R.layout.fragment_appsearch_listitem, searchResultItems);
        searchResultList.setAdapter(appSearchAdapter);

        searchResultList.setOnItemClickListener(new AdapterView.OnItemClickListener() {
            @Override
            public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
                AppSearchListItem item = searchResultItems.get(position);
                if (item == null)
                    return;
                navigateToResult(item);
            }
        });

        return oFragmentView;
    }

    public void executeSearch(boolean reset) {
        if(sSearchQuery == null)
            parseIstanceState(null);
        Log.i("AppSearchInternal", "Action: executeSearch "+sSearchQuery);
        if(AppContext == null)
            AppContext = CampusAppContext.getInstance();
        if(bQueryExecuted && !reset)
            return;
        bQueryExecuted = true;

        searchResultItems.clear();

        SearchIndices[] indices = AppContext.getDatabaseManager().performSearchRequest(sSearchQuery, 40);
        for(int i = 0; i < indices.length; i++) {
            SearchTarget target = new SearchTarget(indices[i].getTarget());
            AppSearchListItem item = new AppSearchListItem(indices[i].getTitle(), indices[i].getDescription(), target);
            searchResultItems.add(item);
        }
        if(appSearchAdapter != null)
            appSearchAdapter.notifyDataSetChanged();

    }

    private void navigateToResult(AppSearchListItem result) {
        SearchTarget target = result.getTarget();

        target.navigate(AppContext.getNavigationManager());
    }

}
