package de.dhbwloe.campusapp.fragments;


import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Point;
import android.os.Bundle;
import android.os.Handler;
import android.support.v4.app.Fragment;
import android.view.Display;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.ProgressBar;

import java.util.Date;

import de.dhbwloe.campusapp.CampusApp;
import de.dhbwloe.campusapp.CampusAppFragment;
import de.dhbwloe.campusapp.R;
import de.dhbwloe.campusapp.mensaplan.MensaplanManager;
import de.dhbwloe.campusapp.mensaplan.MensaplanManagerInterface;
import de.dhbwloe.campusapp.news.NewsManager;
import de.dhbwloe.campusapp.news.NewsManagerInterface;
import de.dhbwloe.campusapp.search.SearchIndices;
import de.dhbwloe.campusapp.vorlesungen.CalendarManager;
import de.dhbwloe.campusapp.vorlesungen.CalendarManagerInterface;
import de.dhbwloe.campusapp.vorlesungen.CourseEvent;
import de.dhbwloe.campusapp.vorlesungen.CourseGroup;

/**
 * A simple {@link Fragment} subclass.
 */
public class SplashScreen extends CampusAppFragment {
    private ProgressBar splashProgress;
    private int progressCounter;
    private Handler timerHandler = new Handler();
    private Runnable timerRunnable;

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);


    }

    @Override
    public void onSaveInstanceState(Bundle savedInstanceState) {
        timerHandler.removeCallbacksAndMessages(null);
        super.onSaveInstanceState(savedInstanceState);
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        View view = inflater.inflate(R.layout.fragment_splashscreen, container, false);

        splashProgress = (ProgressBar)view.findViewById(R.id.splashProgress);
        splashProgress.setMax(20);
        splashProgress.setProgress(0);
        progressCounter = 0;

        AppContext.setTitle("DHBW Lörrach");

        ImageView splashImage = (ImageView)view.findViewById(R.id.splashImage);
        BitmapFactory.Options dimensions = new BitmapFactory.Options();
        Bitmap mBitmap = BitmapFactory.decodeResource(getResources(), R.drawable.dhbw_campus_hd, dimensions);
        int height = dimensions.outHeight;
        int width =  dimensions.outWidth;
        Display display = AppContext.getMainActivity().getWindowManager().getDefaultDisplay();
        Point size = new Point();
        display.getSize(size);
        float scaleX = (float)size.x / (float)width;
        float scaleY = (float)size.y / (float)height;
        float scale = Math.max(scaleX, scaleY);
        int newWidth = (int)(width*scale);
        int newHeight = (int)(height*scale);
        Bitmap newBitmap = Bitmap.createScaledBitmap(mBitmap, newWidth, newHeight, true);
        splashImage.setImageBitmap(newBitmap);

        timerRunnable = new Runnable() {
            @Override
            public void run() {
                progressCounter++;
                splashProgress.setProgress(progressCounter);
                long now = (new Date()).getTime()/1000;
                switch(progressCounter) {
                    case 1:
                        AppContext.getDatabaseManager().initializeDatabase();
                        break;
                    case 2:
                        AppContext.addDefaultSearchIndexes();
                        break;
                    case 3:
                        String startCounter = AppContext.getDatabaseManager().getRuntimeCache("AppStartCounter");
                        if(startCounter == null || Integer.parseInt(startCounter) == 0) {
                            AppContext.getNavigationManager().navigatePage("FirstRun", null, false);
                            return;
                        }
                        AppContext.getDatabaseManager().setRuntimeCache("AppStartCounter", Integer.toString(Integer.parseInt(startCounter) + 1));
                        break;
                    case 4:
                        String lastVLMFullSyncStr = AppContext.getDatabaseManager().getRuntimeCache("LastVLMFullSync");
                        String lastVLMPartialSyncStr = AppContext.getDatabaseManager().getRuntimeCache("LastVLMPartialSync");
                        long lastVLMFullSync, lastVLMPartialSync;
                        if(lastVLMFullSyncStr != null)
                            lastVLMFullSync = Long.parseLong(lastVLMFullSyncStr);
                        else
                            lastVLMFullSync = 0;
                        if(lastVLMPartialSyncStr != null)
                            lastVLMPartialSync = Long.parseLong(lastVLMPartialSyncStr);
                        else
                            lastVLMPartialSync = 0;

                        String kursTag = AppContext.getDatabaseManager().getRuntimeCache("CourseName");
                        if(kursTag == null || kursTag.isEmpty())
                            break;
                        CalendarManager vpm = new CalendarManager(AppContext, kursTag);

                        if(lastVLMFullSync == 0 || now - lastVLMFullSync > (86400 * 14)) { // full sync every 14 days
                            vpm.performFullSynchronisation(new CalendarManagerInterface() {
                                @Override
                                public void onCalendarUpdateDone() {
                                    long now = (new Date()).getTime() / 1000;
                                    AppContext.getDatabaseManager().setRuntimeCache("LastVLMFullSync", Long.toString(now));
                                    AppContext.getDatabaseManager().setRuntimeCache("LastVLMPartialSync", Long.toString(now));
                                    timerHandler.postDelayed(timerRunnable, 100);
                                }

                                @Override
                                public void onCalendarUpdateFail(String errorMessage) {
                                    timerHandler.postDelayed(timerRunnable, 100);
                                }
                                @Override
                                public SearchIndices onGenerateCalendarSearchIndices(CourseEvent event) {
                                    CourseGroup group = event.getCourseGroup();
                                    SearchIndices indices = new SearchIndices("Vorlesungsplan#Group"+group.getGroupId(), false);
                                    indices.setUpdateTime(event.getEventFrom());
                                    indices.setTarget("#Vorlesungsplan#groupid=" + group.getGroupId());
                                    indices.setTitle("Vorlesungsplan " + event.getCourseName());
                                    indices.setDescription("Vorlesung " + event.getEventTitle());
                                    indices.addKeyWord(event.getGroupTitle());
                                    indices.addKeyWord(event.getEventLocation());
                                    return indices;
                                }
                            });
                        }
                        else if(lastVLMPartialSync == 0 || now - lastVLMPartialSync > (86400)) { // partial sync every day
                            vpm.performFastSynchronisation(new CalendarManagerInterface() {
                                @Override
                                public void onCalendarUpdateDone() {
                                    long now = (new Date()).getTime()/1000;
                                    AppContext.getDatabaseManager().setRuntimeCache("LastVLMPartialSync", Long.toString(now));
                                    timerHandler.postDelayed(timerRunnable, 100);
                                }
                                @Override
                                public void onCalendarUpdateFail(String errorMessage) {
                                    timerHandler.postDelayed(timerRunnable, 100);
                                }
                                @Override
                                public SearchIndices onGenerateCalendarSearchIndices(CourseEvent event) {
                                    CourseGroup group = event.getCourseGroup();
                                    SearchIndices indices = new SearchIndices("Vorlesungsplan#Group"+group.getGroupId(), false);
                                    indices.setUpdateTime(event.getEventFrom());
                                    indices.setTarget("#Vorlesungsplan#groupid=" + group.getGroupId());
                                    indices.setTitle(AppContext.getResString(R.string.search_vorlesungsplan_group_title, event.getCourseName()));
                                    indices.setDescription(event.getEventTitle());
                                    indices.addKeyWord(event.getGroupTitle());
                                    indices.addKeyWord(event.getEventLocation());
                                    return indices;
                                }
                            });
                        }
                        else
                            break;
                        return;
                    case 5:
                        String lastMPMSyncStr = AppContext.getDatabaseManager().getRuntimeCache("LastMPSync");
                        long lastMPMSync;
                        if(lastMPMSyncStr != null)
                            lastMPMSync = Long.parseLong(lastMPMSyncStr);
                        else
                            lastMPMSync = 0;

                        MensaplanManager mpm = new MensaplanManager(AppContext);

                        if(lastMPMSync == 0 || now - lastMPMSync > (86400 * 2)) { //sync every 2 days
                            mpm.performSynchronisation(new MensaplanManagerInterface() {
                                @Override
                                public void onMensaplanUpdateDone() {
                                    long now = (new Date()).getTime() / 1000;
                                    AppContext.getDatabaseManager().setRuntimeCache("LastMPSync", Long.toString(now));
                                    timerHandler.postDelayed(timerRunnable, 100);
                                }

                                @Override
                                public void onMensaplanUpdateFail(String errorMessage) {
                                    timerHandler.postDelayed(timerRunnable, 100);
                                }
                            });
                        }
                        else
                            break;
                        return;
                    case 6:
                    case 7:
                        final String syncSource = (progressCounter == 6 ? "DHBW" : "STUV");
                        String lastNewsSyncStr = AppContext.getDatabaseManager().getRuntimeCache("LastNewsSync_"+syncSource);
                        long lastNewsSync;
                        if(lastNewsSyncStr != null)
                            lastNewsSync = Long.parseLong(lastNewsSyncStr);
                        else
                            lastNewsSync = 0;

                        NewsManager nm = new NewsManager(AppContext, syncSource);

                        if(lastNewsSync == 0 || now - lastNewsSync > (86400 * 1)) { //sync every day
                            nm.performSynchronisation(new NewsManagerInterface() {
                                @Override
                                public void onNewsUpdateDone() {
                                    long now = (new Date()).getTime() / 1000;
                                    AppContext.getDatabaseManager().setRuntimeCache("LastNewsSync_"+syncSource, Long.toString(now));
                                    timerHandler.postDelayed(timerRunnable, 100);
                                }

                                @Override
                                public void onNewsUpdateFail(String errorMessage) {
                                    timerHandler.postDelayed(timerRunnable, 100);
                                }
                            });
                        }
                        else
                            break;
                        return;
                    case 8:
                        String lastStuvSyncStr = AppContext.getDatabaseManager().getRuntimeCache("LastStuvSync");
                        long lastStuvSync;
                        if(lastStuvSyncStr != null)
                            lastStuvSync = Long.parseLong(lastStuvSyncStr);
                        else
                            lastStuvSync = 0;

                        final String calendarName = "STUV";
                        CalendarManager stuvsyncmgr = new CalendarManager(AppContext, calendarName);

                        if(lastStuvSync == 0 || now - lastStuvSync > (86400 * 3)) { // full sync every 3 days
                            stuvsyncmgr.performFullSynchronisation(new CalendarManagerInterface() {
                                @Override
                                public void onCalendarUpdateDone() {
                                    long now = (new Date()).getTime() / 1000;
                                    AppContext.getDatabaseManager().setRuntimeCache("LastStuvSync", Long.toString(now));
                                    timerHandler.postDelayed(timerRunnable, 100);
                                }

                                @Override
                                public void onCalendarUpdateFail(String errorMessage) {
                                    timerHandler.postDelayed(timerRunnable, 100);
                                }

                                @Override
                                public SearchIndices onGenerateCalendarSearchIndices(CourseEvent event) {
                                    CourseGroup group = event.getCourseGroup();
                                    SearchIndices indices = new SearchIndices("Vorlesungsplan#Group"+group.getGroupId(), false);
                                    indices.setUpdateTime(event.getEventFrom());
                                    indices.setTarget("#News#showevent=" + group.getGroupId() + "&course=" + calendarName);
                                    indices.setTitle(calendarName+" Event: " + event.getCourseName());
                                    indices.setDescription(event.getEventTitle());
                                    indices.addKeyWord(event.getGroupTitle());
                                    indices.addKeyWord(event.getEventLocation());

                                    return null;
                                }
                            });
                        }
                        else
                            break;
                        return;

                    // some more tasks to do here?

                    case 20:
                        ((CampusApp)AppContext.getMainActivity()).loadMainUi();
                        AppContext.getNavigationManager().navigatePage("Dashboard", null, false);
                        return;
                }
                timerHandler.postDelayed(timerRunnable, 100);
            }
        };

        return view;
    }

    @Override
    public void onResume() {
        super.onResume();
        timerHandler.postDelayed(timerRunnable, 500);
    }

}
