package de.dhbwloe.campusapp.mensaplan;

import android.util.Log;

import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.text.DateFormat;
import java.text.NumberFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Locale;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import de.dhbwloe.campusapp.CampusAppContext;
import de.dhbwloe.campusapp.network.XmlEntry;
import de.dhbwloe.campusapp.network.XmlRequestHelper;
import de.dhbwloe.campusapp.search.SearchIndices;

/**
 * Created by pk910 on 22.01.2016.
 */
public class MensaplanManager extends XmlRequestHelper {
    private CampusAppContext AppContext;
    private boolean bRequestRunning = false;
    private ArrayList<MensaplanManagerInterface> aCallbackInterfaces = new ArrayList<MensaplanManagerInterface>();

    public MensaplanManager(CampusAppContext context) {
        AppContext = context;
    }

    public void performSynchronisation(MensaplanManagerInterface callback) {
        aCallbackInterfaces.add(callback);
        if(bRequestRunning)
            return;

        bRequestRunning = true;
        String mensaplanUrl = "http://www.swfr.de/index.php?id=1400&type=98&tx_swfrspeiseplan_pi1[apiKey]=c3841e89a2c8c301b890723ecdb786ad&tx_swfrspeiseplan_pi1[ort]=677";
        requestXmlFromWeb(mensaplanUrl, "plan", "tagesplan");
    }

    private long getTimeFromDateString(String timestamp) {
        // 01.03.2016
        if(timestamp == null)
            return 0;
        DateFormat df = new SimpleDateFormat("dd.MM.yyyy", Locale.ENGLISH);
        try {
            Date result =  df.parse(timestamp);
            return result.getTime()/1000;
        } catch (ParseException e) {
            return 0;
        }
    }

    private int getPriceFromString(String pricestr) {
        // 2,90€( pro 100g)?
        if(pricestr == null)
            return 0;
        Pattern r = Pattern.compile("^([0-9,]+)€( pro [0-9]+[a-z]+)?$");
        Matcher m = r.matcher(pricestr);
        if (m.find()) {
            NumberFormat nf = NumberFormat.getInstance(Locale.GERMAN);
            try {
                Number result = nf.parse(m.group(1));
                return (int)(result.doubleValue()*100);
            } catch (ParseException e) {
                return 0;
            }
        } else
            return 0;
    }

    @Override
    protected void onXmlReceived(List<XmlEntry> entries) {
        ArrayList<SearchIndices> newIndices = new ArrayList<SearchIndices>();

        Log.i("MPMSync", "Received Mensaplan XML");
        for(XmlEntry entry : entries) {
            Log.i("MPMSync", "Parse Tagesplan: "+entry.getAttribute("datum"));
            long plandate = getTimeFromDateString(entry.getAttribute("datum"));
            if(plandate == 0)
                continue;

            XmlEntry[] menues = XmlEntry.FindXmlEntriesByName(entry, "menue");
            for(XmlEntry menue : menues) {
                XmlEntry centry;
                String menuname = menue.getAttribute("art");
                Log.i("MPMSync", "Parse menu "+menuname);
                if(menuname == null) continue;

                String additional = menue.getAttribute("zusatz");

                centry = XmlEntry.FindXmlEntryByName(menue, "name");
                String name = (centry == null ? null : centry.getValue());

                centry = XmlEntry.FindXmlEntryByName(menue, "nameMitUmbruch");
                String nameHtml = (centry == null ? null : centry.getValue());

                centry = XmlEntry.FindXmlEntryByName(menue, "kennzeichnungen");
                String notes = (centry == null ? null : centry.getValue());

                int priceArray[] = new int[4];

                centry = XmlEntry.FindXmlEntryByName(menue, "studierende");
                priceArray[0] = (centry == null ? 0 : getPriceFromString(centry.getValue()));
                centry = XmlEntry.FindXmlEntryByName(menue, "angestellte");
                priceArray[1] = (centry == null ? 0 : getPriceFromString(centry.getValue()));
                centry = XmlEntry.FindXmlEntryByName(menue, "gaeste");
                priceArray[2] = (centry == null ? 0 : getPriceFromString(centry.getValue()));
                centry = XmlEntry.FindXmlEntryByName(menue, "schueler");
                priceArray[3] = (centry == null ? 0 : getPriceFromString(centry.getValue()));

                MensaTagesplan tagesplan = new MensaTagesplan(plandate, menuname, 0, name, nameHtml, additional, notes, priceArray[0], priceArray[1], priceArray[2], priceArray[3]);
                tagesplan.calculateChkSum();

                AppContext.getDatabaseManager().updateMensaTagesplan(tagesplan);

                if(tagesplan.getIsNew(true)) {
                    SearchIndices indices = new SearchIndices("MensaTagesplan#"+tagesplan.getPlanDate()+"|"+tagesplan.getMenuName(), false);
                    indices.setUpdateTime(tagesplan.getPlanDate());
                    try {
                        indices.setTarget("#Mensa#showdate=" + tagesplan.getPlanDate()+"&showmenu=" + URLEncoder.encode(tagesplan.getMenuName(), "UTF-8"));
                    } catch (UnsupportedEncodingException e) {
                        indices.setTarget("#Mensa#showdate=" + tagesplan.getPlanDate());
                    }
                    indices.setTitle("Mensaplan " + tagesplan.getFormatedDate());
                    indices.setDescription(tagesplan.getMenuName() + ": " + tagesplan.getName());
                    indices.addKeyWord(tagesplan.getName());
                    indices.addKeyWord(tagesplan.getNotes());
                    newIndices.add(indices);
                }
            }
        }

        SearchIndices[] newIndicesArr = new SearchIndices[newIndices.size()];
        newIndicesArr = newIndices.toArray(newIndicesArr);
        AppContext.addSearchIndices(newIndicesArr);

        for(MensaplanManagerInterface callback : aCallbackInterfaces) {
            callback.onMensaplanUpdateDone();
        }
        aCallbackInterfaces.clear();
        bRequestRunning = false;
    }

    @Override
    protected void onXmlRequestFail(int statusCode, String errorMessage) {
        for(MensaplanManagerInterface callback : aCallbackInterfaces) {
            callback.onMensaplanUpdateFail("error " + statusCode + ": " + errorMessage);
        }
        aCallbackInterfaces.clear();
        bRequestRunning = false;
    }
}
