package de.dhbwloe.campusapp.news;

import android.util.Base64;
import android.util.Log;

import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Locale;

import de.dhbwloe.campusapp.CampusAppContext;
import de.dhbwloe.campusapp.mensaplan.MensaplanManagerInterface;
import de.dhbwloe.campusapp.network.XmlEntry;
import de.dhbwloe.campusapp.network.XmlRequestHelper;
import de.dhbwloe.campusapp.search.SearchIndices;

/**
 * Created by pk910 on 22.01.2016.
 */
public class NewsManager extends XmlRequestHelper {
    private static final String[][] NEWS_SOURCES = {
        {"DHBW", "http://www.dhbw-loerrach.de/index.php?id=59&type=100"},
        {"STUV", "http://stuv-loerrach.de/feed"},
    };


    private CampusAppContext AppContext;
    private boolean bRequestRunning = false;
    private ArrayList<NewsManagerInterface> aCallbackInterfaces = new ArrayList<NewsManagerInterface>();
    private String[] source;

    public NewsManager(CampusAppContext context, String source) {
        AppContext = context;

        for(String[] src : NEWS_SOURCES) {
            if(src[0].equalsIgnoreCase(source)) {
                this.source = src;
            }
        }
    }

    public void performSynchronisation(NewsManagerInterface callback) {
        aCallbackInterfaces.add(callback);
        if(bRequestRunning || this.source == null)
            return;

        bRequestRunning = true;
        requestXmlFromWeb(this.source[1], "channel", "item");
    }

    private long getTimeFromDateString(String timestamp) {
        // Thu, 08 Oct 2015 08:00:34 +0000
        if(timestamp == null)
            return 0;
        DateFormat df = new SimpleDateFormat("E, dd MMM yyyy HH:mm:ss Z", Locale.ENGLISH);
        try {
            Date result =  df.parse(timestamp);
            return result.getTime()/1000;
        } catch (ParseException e) {
            return 0;
        }
    }

    @Override
    protected void onXmlReceived(List<XmlEntry> entries) {
        ArrayList<SearchIndices> newIndices = new ArrayList<SearchIndices>();

        Log.i("NMSync", "Received News XML");
        for(XmlEntry entry : entries) {
            Log.i("NMSync", "Parse News");
            XmlEntry centry;

            centry = XmlEntry.FindXmlEntryByName(entry, "pubDate");
            long time = (centry == null ? 0 : getTimeFromDateString(centry.getValue()));
            if(time == 0)
                continue;

            centry = XmlEntry.FindXmlEntryByName(entry, "link");
            String link = (centry == null ? null : centry.getValue());
            centry = XmlEntry.FindXmlEntryByName(entry, "uuid");
            String uuid = (centry == null ? null : centry.getValue());

            String uniqueid;
            if(uuid == null) {
                if(link == null)
                    continue;
                byte[] data = new byte[0];
                try {
                    data = link.getBytes("UTF-8");
                } catch (UnsupportedEncodingException e) {
                    continue;
                }
                uniqueid = Base64.encodeToString(data, Base64.DEFAULT);
            } else {
                byte[] data = new byte[0];
                try {
                    data = uuid.getBytes("UTF-8");
                } catch (UnsupportedEncodingException e) {
                    continue;
                }
                uniqueid = Base64.encodeToString(data, Base64.DEFAULT);
            }

            centry = XmlEntry.FindXmlEntryByName(entry, "title");
            String title = (centry == null ? null : centry.getValue());

            centry = XmlEntry.FindXmlEntryByName(entry, "description");
            String summary = (centry == null ? null : centry.getValue());

            centry = XmlEntry.FindXmlEntryByName(entry, "content");
            String content = (centry == null ? null : centry.getValue());

            centry = XmlEntry.FindXmlEntryByName(entry, "creator");
            String creator = (centry == null ? null : centry.getValue());
            if(creator != null) {
                if(content != null)
                    content = creator + ": " + content;
                else
                    content = creator;
            }

            StringBuilder categories = new StringBuilder();
            XmlEntry[] centries = XmlEntry.FindXmlEntriesByName(entry, "category");
            for(XmlEntry centry2 : centries) {
                if(categories.length() > 0)
                    categories.append(",");
                categories.append(centry2.getValue());
            }

            NewsItem newsitem = new NewsItem(0, source[0], time, uniqueid, 0, title, summary, content, link, categories.toString());
            newsitem.calculateChkSum();

            AppContext.getDatabaseManager().updateNewsItem(newsitem);
            if(newsitem.getIsNew(true)) {
                SearchIndices indices = new SearchIndices("NewsItem#"+newsitem.getId(), false);
                indices.setTarget("#News#shownews=" + newsitem.getId());
                indices.setTitle(newsitem.getSource() + " News (" + newsitem.getFormatedDate() + ")");
                indices.setDescription(newsitem.getTitle() + ": " + newsitem.getSummary());
                indices.setUpdateTime(newsitem.getTime());
                indices.addKeyWord(newsitem.getTitle());
                indices.addKeyWord(newsitem.getSummary());
                indices.addKeyWord(newsitem.getContent());
                newIndices.add(indices);
            }
        }

        SearchIndices[] newIndicesArr = new SearchIndices[newIndices.size()];
        newIndicesArr = newIndices.toArray(newIndicesArr);
        AppContext.addSearchIndices(newIndicesArr);

        for(NewsManagerInterface callback : aCallbackInterfaces) {
            callback.onNewsUpdateDone();
        }
        aCallbackInterfaces.clear();
        bRequestRunning = false;
    }

    @Override
    protected void onXmlRequestFail(int statusCode, String errorMessage) {
        for(NewsManagerInterface callback : aCallbackInterfaces) {
            callback.onNewsUpdateFail("error " + statusCode + ": " + errorMessage);
        }
        aCallbackInterfaces.clear();
        bRequestRunning = false;
    }
}
