package de.dhbwloe.campusapp.nfcreader;

import android.nfc.tech.IsoDep;

import de.dhbwloe.campusapp.nfcreader.MifareDESFire.CommunicationError;
import de.dhbwloe.campusapp.nfcreader.MifareDESFire.DesfireManufacturingData;
import de.dhbwloe.campusapp.nfcreader.MifareDESFire.MifareDESFireWrapper;
import de.dhbwloe.campusapp.nfcreader.MifareDESFire.ValueFileSettings;

import java.text.DecimalFormat;

/**
 * Created by stefan on 20.01.16.
 */
public class DHBWCard {

//    /* Commands */
//    public static final byte GET_VERSION_INFO    = (byte) 0x60;
//    public static final byte GET_APPLICATION_DIRECTORY = (byte) 0x6A;
//    public static final byte GET_ADDITIONAL_FRAME      = (byte) 0xAF;
//    public static final byte SELECT_APPLICATION        = (byte) 0x5A;
//    public static final byte READ_DATA                 = (byte) 0xBD;
//    public static final byte READ_RECORD               = (byte) 0xBB;
//    public static final byte GET_FILES                 = (byte) 0x6F;
//    public static final byte GET_FILE_SETTINGS         = (byte) 0xF5;
//
//    public static final byte INIT_AUTH         = (byte) 0x0a;
//    public static final byte FINISH_AUTH         = (byte) 0xAF;
//
//    public static final byte GET_KEY_VERSION = (byte) 0x64;

    private MifareDESFireWrapper nfcCard;

    public DHBWCard(IsoDep tag) {
        this.nfcCard = new MifareDESFireWrapper(tag);
    }


    /**
     * Reads the current uniqueid on the card.
     * Throws a UndefinedResponseException if the card returns an unexpected response (e.g. used another card)
     *
     * @return Returns the unique id
     * @throws UndefinedResponseException
     */
    public int readUniqueId() throws UndefinedResponseException, CommunicationError {
        // Select application for balance
        DesfireManufacturingData manufacturingData = nfcCard.readManufacturingData();
        return manufacturingData.uid;
    }

    /**
     * Reads the current balance on the card.
     * Throws a UndefinedResponseException if the card returns an unexpected response (e.g. used another card)
     *
     * @return Returns the balance
     * @throws UndefinedResponseException
     */
    public int readBalance() throws UndefinedResponseException, CommunicationError {
        // Select application for balance
        nfcCard.selectApplication("5F 84 15");
        int value = nfcCard.readValueFile("01");
        return value / 10;
    }


    /**
     * Reads the last transaction by reading the limited credit value
     * @return Returns the last transaction of the card
     * @throws UndefinedResponseException
     */
    public int readLastTransaction() throws UndefinedResponseException, CommunicationError {
        nfcCard.selectApplication("5F 84 15");
        ValueFileSettings valueFileSettings = (ValueFileSettings) nfcCard.readFileSettings("01");
        return valueFileSettings.getLimitedCreditValue() / 10;
    }
}
